/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg.springbootmockmvc;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.util.List;
import java.util.Map;
import static org.assertj.core.api.AssertionsForClassTypes.assertThat;
import static org.hamcrest.Matchers.hasSize;
import static org.hamcrest.Matchers.is;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 *
 * @author omozegieaziegbe
 */
@WebMvcTest(ProductController.class)
public class ProductControllerTest {

    @Autowired
    private MockMvc mockMvc;
    
    // ObjectMapper for converting objects to JSON
    @Autowired
    private ObjectMapper objectMapper;
    
    @Test
    public void testSimpleEndPoint() throws Exception{
     mockMvc.perform(get("/api/product")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.name").value("Real World Java EE Patterns"))
                .andDo(print());
    }

    @Test
    public void testListOfStrings() throws Exception {
        mockMvc.perform(get("/api/strings")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$", hasSize(3)))
                .andExpect(jsonPath("$[0]", is("The Book of CSS3")))
                .andExpect(jsonPath("$[1]", is("Core HTML5 Canvas")))
                .andExpect(jsonPath("$[2]", is("Pro JavaScript for Web Apps")))
                .andDo(print());
    }

    @Test
    public void testStringMap() throws Exception {
        mockMvc.perform(get("/api/stringmap")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.1", is("Pro JavaScript for Web Apps")))
                .andExpect(jsonPath("$.2", is("JavaScript Enlightenment")))
                .andExpect(jsonPath("$.3", is("Real World Java EE Patterns")))
                .andDo(print());
    }


    @Test
    public void testProductList() throws Exception {

         //perform a GET request to "/api/products" and retrieve the response
        MvcResult mvcResult = mockMvc.perform(get("/api/products")
                .accept(MediaType.APPLICATION_JSON)).andReturn();

        objectMapper = new ObjectMapper();
        
        // Convert the JSON response to a List<Product>
        String responseContent = mvcResult.getResponse().getContentAsString();
        List<Product> productList = objectMapper.readValue(responseContent, new TypeReference<List<Product>>() {});

        // Assert the size of the list
        assertThat(productList.size()).isEqualTo(3);

         //perform additional assertions on the JSON response
        mockMvc.perform(get("/api/products")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$", hasSize(3)))
                .andExpect(jsonPath("$[0].name", is("The Book of CSS3")))
                .andExpect(jsonPath("$[0].price", is(50.00)))
                .andExpect(jsonPath("$[1].name", is("JavaScript and JQuery")))
                .andExpect(jsonPath("$[1].price", is(70.00)))
                .andExpect(jsonPath("$[2].name", is("Core HTML5 Canvas")))
                .andExpect(jsonPath("$[2].price", is(90.00)))
                .andDo(print());

    }

    @Test
    public void testProductMap() throws Exception {
        
        //perform a GET request to "/api/productmap" and retrieve the response
        MvcResult result = mockMvc.perform(get("/api/productmap"))
                .andExpect(status().isOk())
                .andReturn();

         //Convert the JSON response to a Map<String, Product>
        String responseContent = result.getResponse().getContentAsString();
        Map<String, Product> productMap = objectMapper.readValue(responseContent, new TypeReference<Map<String, Product>>() {});

        // Assert the size of the map
        assertThat(productMap.size()).isEqualTo(2);
        
        //perform additional assertions on the JSON response, treating it as a map
        mockMvc.perform(get("/api/productmap")
                .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.1.name", is("Laptop")))
                .andExpect(jsonPath("$.1.price", is(999.99)))
                .andExpect(jsonPath("$.2.name", is("Smartphone")))
                .andExpect(jsonPath("$.2.price", is(599.99)))
                .andDo(print());
    }

}
